#' Simulate a phenotype
#' 
#' @param X A genotype matrix (rows = taxa, columns = markers).
#' @param h2 The heritability of the simulated trait.
#' @param alpha The p parameter in an approximated geometric distribution
#' @NQTN The number markers contributing to the phenotype
#' @a2 The proportion of non-additive interactive variance (FIXED at 0 for our current uses)
#' @seed A seed number to fix the randomization (if desired)
#' @return A list of the marker effects (1*NQTN), the phenotype (n*1), the combined additive effects (n*1), the residual effects (n*1), the additive marker indices (1*m), and the interaction marker positions (1*nint)

G2P <- function(X = NULL,h2 = 0.5,alpha = 1,NQTN = 10,distribution = "normal",a2=0){

  n=nrow(X) # number of taxa/samples
  m=ncol(X) # number of markers
  #Sampling QTN
  QTN.position=sample(m,NQTN,replace=F) # Randomly sample markers
  SNPQ=as.matrix(X[,QTN.position]) # Extract the selected markers
  QTN.position
  #QTN effects
  if(distribution=="normal")
  {addeffect=rnorm(NQTN,0,1)
  }else
  {addeffect=alpha^(1:NQTN)} # Simulate with the pseudo-geometric distribution (THIS COULD BE ANY DISTRIBUTION YOU WANT)
  #Simulate phenotype
  effect=SNPQ%*%addeffect # Use matrix multiplication to get the total effect
  effectvar=var(effect)
  
  #Interaction (DOESN'T appear to be used....)
  cp=0*effect
  nint=4 # The number of interactions appears to be fixed at 4. MAGIC NUMBER???
  if(a2>0&NQTN>=nint){
    for(i in nint:nint){
      Int.position=sample(NQTN,i,replace=F)
      cp=apply(SNPQ[,Int.position],1,prod)
    }
    print(dim(cp))
    cpvar=var(cp)
    intvar=(effectvar-a2*effectvar)/a2
    if(is.na(cp[1]))stop("something wrong in simulating interaction")
    
    if(cpvar!=0){
      print(c(effectvar,intvar,cpvar,var(cp),a2))
      print(dim(cp))
      cp=cp/sqrt(cpvar)
      cp=cp*sqrt(intvar)
      effectvar=effectvar+intvar
    }else{cp=0*effect}
  }
  
  residualvar=(effectvar-h2*effectvar)/h2 # Calculate the variance of the residual (scaled to the already simulated effects)
  residual=rnorm(n,0,sqrt(residualvar)) # Sample residual from a normal distribution
  y=effect+residual+cp # Combine simulated effects, residual, and interations
  return(list(addeffect = addeffect, y=y, add = effect, residual = residual, QTN.position=QTN.position, SNPQ=SNPQ,int=cp))
}

#' Plot statistics for any effect
#' 
#' @param effect A vector of values to plot
#' @param effect_name The effect name to use for the plot title
#' @return A gridExtra arrangement of ggplots
quadplot_stats <- function(effect, effect_name)
{
  add_scatter <- qplot(1:length(effect), effect) +
    labs(x = "Index", y = "Effect")
  
  add_hist <- qplot(effect, bins = 30) +
    labs(x = "Effect", y = "Frequency")
  
  add_box <- qplot(x = factor(alpha), y = effect, geom = "boxplot") +
    labs(x = "Alpha", y = "Effect")
  
  add_ecdf <- ggplot(data.frame(effect), aes(effect)) + stat_ecdf(geom = "step") +
    labs(x = "Additive Effect", y = "Cumulative Density")
  
  return(grid.arrange(add_scatter, 
               add_hist, 
               add_box, 
               add_ecdf, 
               top = paste(effect_name, "Effect Statistics")))
}

#' Generate a Manhattan Plot of a GWAS result
#' 
#' @param marker_map A data.frame containing marker 'rs', 'chr', and 'pos' (should be in the same order as the GWAS results)
#' @param pvals The untransformed p-values from a GWAS test
#' @param QTN_index The indices of any known QTNs
#' @param trait The name of the trait
#' @return A ggplot object Manhattan plot

manhattan_plot <- function(marker_map, pvals, QTN_index = c(), trait = "unknown")
{
  marker_map$pvals <- -log10(pvals) # Add pvalues to the data.frame and log10 transform
  
  marker_map$comb_pos <- marker_map$chr * 1e9 + marker_map$pos
  manhattan_plot <- ggplot(marker_map, aes(x = 1:nrow(marker_map), y = pvals, color = factor(chr))) +
    geom_point() +
    geom_vline(xintercept = QTN_index, color = "red") +
    labs(title = paste("GWAS manhattan plot for trait:", trait),
         y = "-log10(p-value)",
         x = "Marker Position",
         color = "Chromosome")
  
  return(manhattan_plot)
}

#' Generate a Quantile-Quantile Plot of a GWAS result
#' 
#' @param marker_map A data.frame containing marker 'rs', 'chr', and 'pos' (should be in the same order as the GWAS results)
#' @param pvals The untransformed p-values from a GWAS test
#' @param QTN_index The indices of any known QTNs
#' @param trait The name of the trait
#' @return A ggplot object quantile-quantile plot

qq_plot <- function(marker_map, pvals, QTN_index, trait = "unknown")
{
  marker_map$pvals <- -log10(pvals) # Add pvalues to the data.frame and log10 transform
  
  exp_pval_dist <- -log10(runif(nrow(marker_map), 0, 1)) # Sample random p-values from a uniform distribution between 0 and 1
  
  qq_plot <- ggplot(marker_map, aes(x = sort(exp_pval_dist),
                                    y = sort(marker_map$pvals))) +
    geom_point() +
    geom_abline(intercept = 0, slope = 1, color = "red") +
    labs(title = "Q-Q Plot",
         x = "-log10(p-values) Expected",
         y = "-log10(p-values) Observed")
  
  return(qq_plot)
}
